<?php 
# -- BEGIN LICENSE BLOCK ----------------------------------
#
# This file is part of SyntaxHighlightJS, a plugin for Dotclear 2.
# 
# Copyright (c) 2011 Nicolas Sebastien Pierre-Olivier GEORGES
# http://www.nicolas.georges.fr.free.fr/
# xlat@cpan.org
#
# Licensed under the Artistic License.
# A copy of this license is available in Artistic.txt file or at
# http://dev.perl.org/licenses/artistic.html
#
# -- END LICENSE BLOCK ------------------------------------
class SyntaxHighlightJS
{
	#Utility functions
	public static function getLanguages()
	{
		//returns an associative array of available languages as : language-name => [ min;normal ]
		$languages = array( );
		//returns an associative array of available colorization styles as : style-name => [ min;normal ]
		$dir = dirname(__FILE__).'/../shjs/lang/';
		$od = opendir($dir);
		while($f = readdir($od)){
			if(is_file($dir.$f) && substr($f,-3,3)=='.js'){
				//remove "sh_" , ".min" and ".js"
				$lang = str_replace('.min.js','',$f);
				$value = ($lang != $f) ? "min" : "normal";
				$lang = str_replace('.js','',$lang);
				$lang = str_replace('sh_','',$lang);
				$languages[$lang] = join(';', array( $value, $languages[$lang] ) );
			}
		}
		return $languages;
	}
	public static function getStyles()
	{
		$styles = array( );
		//returns an associative array of available colorization styles as : style-name => [ min,normal ]
		$dir = dirname(__FILE__).'/../shjs/css/';
		$od = opendir($dir);
		while($f = readdir($od)){
			if(is_file($dir.$f) && substr($f,-4,4)=='.css'){
				//remove "sh_" , ".min" and ".css"
				$css = str_replace('.min.css','',$f);
				$value = ($css != $f) ? "min" : "normal";
				$css = str_replace('.css','',$css);
				$css = str_replace('sh_','',$css);
				//~ echo "DEBUG: $f => $lang<br/>
				$styles[$css] = join(';', array( $value, $styles[$css] ) );
			}
		}
		return $styles;
	}
	

	#Admin functions
	public static function registerFunc($wiki2xhtml)
	{
		$shjs = new SyntaxHighlightJS;
		foreach( array_keys( $shjs->getLanguages() ) as $lang )
		{
			$wiki2xhtml->registerFunction('macro:['.$lang.']',array('SyntaxHighlightJS','parse'));
		}
	}

	public static function parse($text,$args)
	{
	    global $core;
		$text = trim($text);
		$args = preg_replace("/^(\[(.*)\]$)/","$2",$args);	#Here is our language
		#TODO: 
		#	- put a toolbar here
		#	- make <pre> tag scrollable horizontally ( how to ? )
		$text = str_replace("\t","  ",$text);		#Replace tab by 2 spaces
		$text = str_replace("&","&amp;",$text);
		$text = str_replace("<","&lt;",$text);
		$text = str_replace(">","&gt;",$text);
		//~ $text = preg_replace("/\n/","</li><li>",$text);
		$maxheight = "340px";
		$startnumbering = 1;
		//Okay, the Copy to clipboard method comes from http://steamdev.com/zclip/
		return '<div align="right" style="font-size: 75%;">
[<a onclick="return shjs_switchWrap(this.parentElement);" href="#">switch wrap mode</a>]&nbsp;[<a onclick="return shjs_switchLineNr(this.parentElement);" href="#">switch line numbering</a>]&nbsp;[<a class="shjs-copy-link" href="#">copy</a>]
</div>' . "<pre class=\"sh_$args\" start=\"$startnumbering\" style=\"word-wrap: break-word; max-height: $maxheight;\">$text</pre>\n";
	}

	#Public functions
	public static function publicEntryAfterContent($core, $_ctx)
	{
		#TODO: use a preference to put a theme selector
		if(!$_ctx->exists('posts')){ return; }	#Don't display on post agregation
		if($_ctx->posts->post_type != 'post'){ return; }
		if('post.html' != $_ctx->current_tpl ){ return; }
		//~ $ns = $core->blog->settings->get('syntaxhighlightjs');
		//~ $mutex = $ns->get( "mutex" );
		#TODO: add a blog-preference
		$default_style = "ide-pb";	
		$pluginfile = $core->blog->getQmarkURL().'pf=SyntaxHighlightJS/shjs';
		$shjs = new SyntaxHighlightJS;
		$options = "";
		$styles = array_keys( $shjs->getStyles() );
		sort( $styles );
		foreach( $styles as $style)
		{
			$selected = ($style == $default_style) ? " selected" : "";
			$options .= "<option$selected>$style</option>\n";
		}
		echo "syntax highlighting: <select id=\"shjs_themeSelect\" onchange=\"if(!this.disabled) return shjssw_switchStyle('$pluginfile/css/','.min.css',this);\" disabled>$options</select>";

	}
	
	public static function publicHeadContent($core, $_ctx)
	{
		$pluginfile = $core->blog->getQmarkURL().'pf=SyntaxHighlightJS/shjs';
		#TODO: read from settings for default theme
		#TODO: use a preference to allow cookies on clients (let's them memorize there preference)
		$initialstyle = "ide-pb";
		#MUST HAVE jquery already loaded ! (it must be the case...)
		echo "
<!-- # SyntaxHighlightJS -->
<script type=\"text/javascript\" src=\"$pluginfile/sh_main.js\"></script>
<script type=\"text/javascript\" src=\"$pluginfile/sh_stylesw.js\"></script>
<script type=\"text/javascript\" src=\"$pluginfile/jquery.zclip.js\"></script>
<script type=\"text/javascript\">
//<![CDATA[
function shjsbs_onload_ie_safari_chrome_firefox(){
	if (/loaded|complete/.test(document.readyState) && window['readCookie']) {
		//look for cookie
		var cookie = readCookie('shjs_lastused');
		if(cookie==null) cookie='$initialstyle';
		sh_highlightDocument('$pluginfile/lang/', '.min.js', cookie+'.min', '$pluginfile/css/');
		var select = document.getElementById('shjs_themeSelect');
		select.value = cookie;
		select.disabled = false;
		//Install copy-buffers hooks
		$(\"a.shjs-copy-link\").each( function(){ shjs_copy(this,'$pluginfile/ZeroClipboard.swf') } );
	}
	else{
		setTimeout('shjsbs_onload_ie_safari_chrome_firefox()', 250);
	}
}
shjsbs_onload_ie_safari_chrome_firefox();
//]]>
</script>
";
	}
	
}

//Here is a hack 'main like'
function main()
{
	$c = new SyntaxHighlightJS;
	echo "Style available:\n\t" . join( ", ", array_keys( $c->getStyles() ) ) . "\n\n";
	echo "Languages available:\n\t" . join( ", ", array_keys( $c->getLanguages() ) ) . "\n\n";
}

//~ main( /*  like in C - test and dev prupose only */ );

?>